<?php
/*
	Plugin Name: WooCommerce MoIP Gateway
	Plugin URI: http://www.vibemidia.com.br/woocommerce-moip
	Description: Integração MoIP para WooCommerce, siga as <a href="http://www.vibemidia.com.br/woocommerce-moip" title="instructions of use" target="_blank">instruções de uso</a>. <strong>Adquira o plugin <a href="http://woocommerce.com.br/plugin/pagamento-digital-plugin-para-woocommerce/" target="_new">Pagamento Digital</a>, <a href="http://woocommerce.com.br/plugin/pagseguro-plugin-woocommerce/" target="_new">PagSeguro</a> e <a href="http://woocommerce.com.br/plugin/correios-plugin-para-woocommerce/" target="_new">Correios</a></strong>.
	Version: 1.5
	Author: John-Henrique
	Author URI: http://johnhenrique.com
	Requires at least: 3.1
	Tested up to: 3.5.1
	Text domain: WC_MoIP
*/

/*  Copyright 2011 John-Henrique 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Required functions

if ( ! function_exists( 'woothemes_queue_update' ) )
	require_once( 'woo-includes/woo-functions.php' );
 */
/**
 * Plugin updates
woothemes_queue_update( plugin_basename( __FILE__ ), '3a76a1744ec5dd93d0c27f8581bad000', '18639' );

 */
add_action('plugins_loaded', 'woocomerce_gateway_moip', 0);

function woocomerce_gateway_moip() {

	if ( ! class_exists( 'WC_Payment_Gateway' ) ) return;

	add_filter('woocommerce_payment_gateways', 'add_moip_gateway' );

	class WC_Gateway_Moip extends WC_Payment_Gateway {

		public function __construct() {
			global $woocommerce;

			$this->id   = 'moip';
			$this->method_title = 'MoIP';
			$this->icon   = WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/moip-p.png';
			$this->has_fields  = false;
			$this->liveurl   = 'https://www.moip.com.br';
			$this->testurl   = 'https://desenvolvedor.moip.com.br/sandbox';
			$this->url    = $this->liveurl;

			// Load the form fields.
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();

			// Define user set variables
			$this->title   = $this->settings['title'];
			$this->description  = $this->settings['description'];
			$this->account   = $this->settings['account'];
			$this->token   = $this->settings['token'];
			$this->key   = $this->settings['key'];

			$this->debug  = $this->settings['debug'];
			$this->testmode  = $this->settings['testmode'];

			// Logs
			if ($this->debug=='yes') $this->log = $woocommerce->logger();

			// Enabled?
			if ( $this->testmode == 'yes' ) $this->url = $this->testurl;

			// Valid?
			if ( !$this->is_valid_currency() || !$this->are_credentials_set() )
				$this->enabled = false;


				
				
			add_action('init', array(&$this, 'fix_url_order_received' ));
				
			// Actions
			add_action( 'woocommerce_update_options_payment_gateways_'. $this->id, array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_api_'. strtolower( get_class( $this ) ), array( $this, 'notificacao' ) );

			// Actions
			add_action('woocommerce_receipt_moip', array(&$this, 'receipt_page' ));
			add_action('admin_notices', array(&$this, 'check'));
		}

		/**
		 * Initialise Gateway Settings Form Fields
		 */
		function init_form_fields() {

			$this->form_fields = array(
				'enabled' => array(
					'title' => __( 'Enable/Disable', 'WC_MoIP' ),
					'type' => 'checkbox',
					'label' => __( 'Enable MoIP standard', 'WC_MoIP' ),
					'default' => 'yes'
				),
				'title' => array(
					'title' => __( 'Title', 'WC_MoIP' ),
					'type' => 'text',
					'description' => __( 'This controls the title which the user sees during checkout.', 'WC_MoIP' ),
					'default' => __( 'MoIP', 'WC_MoIP' )
				),
				'description' => array(
					'title' => __( 'Description', 'WC_MoIP' ),
					'type' => 'textarea',
					'description' => __( 'This controls the description which the user sees during checkout.', 'WC_MoIP' ),
					'default' => __("Pay via MoIP you can pay with your credit card, billet and eletronic transfer same you no have a MoIP account.", 'WC_MoIP')
				),
				'account' => array(
					'title' => __( 'MoIP Account', 'WC_MoIP' ),
					'type' => 'text',
					'description' => __( 'Please enter your MoIP account; this is needed in order to take payment.', 'WC_MoIP' ),
					'default' => ''
				),
				'token' => array(
					'title' => __( 'MoIP Token', 'WC_MoIP' ),
					'type' => 'text',
					'description' => sprintf(__( 'Please enter your MoIP token; this is needed in order to take payment. See %sToken of security%s', 'WC_MoIP' ), '<a href="http://labs.moip.com.br/referencia/autenticacao_api/#sandbox" target="_blank">', '</a>'),
					'default' => ''
				),
				'key' => array(
					'title' => __( 'MoIP Key', 'WC_MoIP' ),
					'type' => 'text',
					'description' => __( 'Please enter your MoIP Key; this is needed in order to take payment.', 'WC_MoIP' ),
					'default' => ''
				),
				'testmode' => array(
					'title' => __( 'MoIP sandbox', 'WC_MoIP' ),
					'type' => 'checkbox',
					'description' => __( 'Enable MoIP sandbox', 'WC_MoIP' ),
					'default' => 'yes'
				),
				'debug' => array(
					'title' => __( 'Enable/Disable Log', 'WC_MoIP' ),
					'type' => 'checkbox',
					'description' => __( 'Enable logging (<code>woocommerce/logs/moip.txt</code>)', 'WC_MoIP' ),
					'default' => ''
				),

			);

		} // End init_form_fields()

		/**
		 * Admin Panel Options
		 * - Options for bits like 'title' and availability on a country-by-country basis
		 *
		 * @since 1.0.0
		 */
		public function admin_options() {

?>
	    	<h3><?php echo __('MoIP Gateway', 'WC_MoIP'); ?></h3>
	    	<p><?php echo __('MoIP works by sending the user to MoIP Gateway to enter their payment information.', 'WC_MoIP');?>
	    		<?php echo sprintf(__('This Payment Gateway works only accounts Seller or Professional. See %s documentation%s', 'WC_MoIP'), '<a href="http://labs.moip.com.br/api/" target="_blank">', '</a>'); ?></p>
	    	<table class="form-table">
	    	<?php
			$this->check();


			// Generate the HTML For the settings form.
			$this->generate_settings_html();
?>
			</table><!--/.form-table-->
	    	<?php
		} // End admin_options()

		public function check() {

			if ( !$this->is_valid_currency() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'WC_MoIP' ) ."</strong>: ". __( 'MoIP does not support your store\'s currency. You need to select the currency of Brazil Real.', 'WC_MoIP' ) ."</p>
				</div>";
			endif;

			if ( ! $this->are_credentials_set() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'WC_MoIP' ) ."</strong>: ". __( 'You must give the token of your account email.', 'WC_MoIP' ) ."</p>
				</div>";
			endif;
		}


		/**
		 * There are no payment fields for moip, but we want to show the description if set.
		 **/
		function payment_fields() {
			if ($this->description) echo wpautop(wptexturize($this->description));
		}

		/**
		 * Check if MoIP can be used with the store's currency.
		 * For now only work with Real of Brazil, but...
		 *
		 * @since 1.0
		 */
		function is_valid_currency() {
			if (get_option('woocommerce_currency') == "BRL" ) {
				return true;
			}


			return false;
		}

		/**
		 * Check if MoIP Credentials are set
		 *
		 * @since 1.0
		 */
		function are_credentials_set() {
			if ( empty( $this->account ) || empty( $this->token ) )
				return false;
			else
				return true;
		}

		/**
		 * Generate the MoIP button link
		 **/
		public function generate_moip_form( $order_id ) {
			global $woocommerce;
			$order = new WC_Order( $order_id );

			$js = '
			jQuery(function(){
				jQuery("body").block(
					{
						message: "<img src=\"'.esc_url( $woocommerce->plugin_url() ).'/assets/images/ajax-loader.gif\" alt=\"'. __("Redirecting...", 'WC_MoIP') .'\" style=\"float:left; margin-right: 10px;\" />'.__('Thank you for your order. We are now redirecting you to MoIP to make payment.', 'WC_MoIP').'",
						overlayCSS:
						{
							background: "#fff",
							opacity: 0.6
						},
						css: {
					        padding:        20,
					        textAlign:      "center",
					        color:          "#555",
					        border:         "3px solid #aaa",
					        backgroundColor:"#fff",
					        cursor:         "wait",
					        lineHeight:		"32px"
					    }
					});
				jQuery("#submit_moip_payment_form").click();
			});';
			

			if ( function_exists( 'wc_enqueue_js' ) ) {
				wc_enqueue_js( $js );
			} else {
				$woocommerce->add_inline_js( $js );
			}
			

			// HTML to payment
			$html = '<form action="'.( "https://www.moip.com.br/PagamentoMoIP.do" ).'" method="post" id="moip_payment_form">
						'. $this->form_html( $order ) .'
						<input type="submit" class="button-alt" id="submit_moip_payment_form" value="'.__('Pay via moip', 'WC_MoIP').'" /> <a class="button cancel" href="'.esc_url( $order->get_cancel_order_url() ).'">'.__('Cancel order &amp; restore cart', 'WC_MoIP').'</a>
					</form>';

			return $html;

		}

		/**
		 * Process the payment and return the result
		 * Executa no checkout 'finalizar compra -> pagar'
		 *
		 **/
		function process_payment( $order_id = 0 ) {
			global $woocommerce;

			$order = new WC_Order( $order_id );

			// no uncomment
			//$order->update_status('on-hold', __('Aguardando confirmação de pagamento do boleto', 'WC_MoIP'));
			
			// clean cart
			$woocommerce->cart->empty_cart();

			// Empty awaiting payment session
			unset( $woocommerce->session->order_awaiting_payment );

			return array(
				'result' 	=> 'success',
				'redirect'	=> $order->get_checkout_payment_url( true )
			);

		}

		/**
		 * receipt_page
		 **/
		function receipt_page( $order ) {
			echo '<p>'.__('Thank you for your order, please click the button below to pay with moip.', 'WC_MoIP').'</p>';
			echo $this->generate_moip_form( $order );
		}

		/**
		 * Ajuda a identificar o status de pagamento
		 * @see https://www.moip.com.br/AdmTransactionManual.do
		 * @param Integer $status status de pagamento retornado
		 * @return Array status de pagamento convertido para o WooCommerce
		 */
		public function status_helper( $status = null, $order = null ) {

			switch ( $status ) {

			case 1:
				$arrStatus['status'] = "completed"; // paga
				$arrStatus['log'] = __('Manual confirmation of payment, Check payment confirmation', 'WC_MoIP');

				// change payment status
				$order->payment_complete();

				break;

			case 3:
				$arrStatus['status'] = "pending"; // Boleto impresso
				$arrStatus['log'] = __('Printed billet, Waiting payment confirmation', 'WC_MoIP'); // aguardando pagamento

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 4:// disponivel
				$arrStatus['status'] = "completed"; // disponivel
				$arrStatus['log'] = __('Payment completed', 'WC_MoIP');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 5: // disputa
				$arrStatus['status'] = "on-hold";
				$arrStatus['log'] = __('Failed on process of negociation (dispute)', 'WC_MoIP');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 6:
				$arrStatus['status'] = "pending"; // aguardando
				$arrStatus['log'] = __('Payment in analysis, Waiting payment confirmation', 'WC_MoIP');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 7:
				$arrStatus['status'] = "refunded"; // devolvida
				$arrStatus['log'] = __('Payment refunded', 'WC_MoIP');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 5:
				$arrStatus['status'] = "cancelled"; // cancelada
				$arrStatus['log'] = __('Order cancelled', 'WC_MoIP');

				// cancel this order
				$order->cancel_order( $order->id );

				break;

				// improvavel mas nao custa prevenir
			default:
				$arrStatus['status'] = 'pending';
				$arrStatus['log'] = __('Payment pending, Waiting payment confirmation', 'WC_MoIP');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

			}

			return $arrStatus;
		}
		
		
		
		/**
		 * This a hack fix to dinamic URL used in MoIP
		 * 
		 * Until this day MoIP no have supports to 
		 * query strings in redirectURL
		 * 
		 * @since 2012-08-18
		 */
		public function fix_url_order_received(){
			global $woocommerce;
			
			
			// if notificationCode and order exists and have a value
			if( isset( $_GET['MoIPretorno'] ) && !empty( $_GET['MoIPretorno'] ) ){
				
			    // this order exists?
			    $order = new WC_Order( (int) $_GET['MoIPretorno'] );
				
		    	/**
		    	 * MoIP no support URLs with query strings
		    	 * This is a simple hack to fix this problem
		    	 * 
		    	 * Verify the transaction code and redirect 
		    	 * to URL with order_id and key
		    	 */
		    	if ( method_exists( $order, 'get_checkout_order_received_url' ) ) {
                	$redirect = $order->get_checkout_order_received_url();
                } else {
                	$redirect = add_query_arg( 'key', $order->order_key, add_query_arg( 'order', $order->id, get_permalink( get_option( 'woocommerce_thanks_page_id' ) ) ) );
                }

		    	wp_redirect( $redirect );
			}
		}
		
		

		function notificacao() {
			global $woocommerce;

			// who are you?
			if ( isset( $_POST['cod_moip'] ) && isset( $_POST['forma_pagamento'] ) ) {

				// short cut
				$order_code  = $_POST['cod_moip'];
				$order_ID   = $_POST['id_transacao'];
				$order_status  = $_POST['status_pagamento'];
				$order_total  = $_POST['valor'];
				$paymentMethod  = $_POST['forma_pagamento'];
				$paymentType  = $_POST['tipo_pagamento'];
				$consumerEmail  = $_POST['email_consumidor'];


				// this order exists?
				$pedido = new WC_Order( (int) $order_ID );


				/**
				 * For security reasons we check if the application
				 * exists and that the total order value is the
				 * same as the amount charged in store
				 *
				 * if any errors occur
				 * kill this process
				 */
				if ( !is_int( $pedido->id ) && $pedido->order_total != $order_total ) {
					$woocommerce->add_error(__('Oops, there was an error during the payment process, please try again.', 'WC_MoIP'));
					header("HTTP/1.0 404 Not Found");
					exit;
				}else {

					// what is this?
					$arrStatus = $this->status_helper( $order_status, $pedido );
					//wp_redirect( $this->get_return_url( $order ) );
					
			    	if ( method_exists( $order, 'get_checkout_order_received_url' ) ) {
	                	$redirect = $pedido->get_checkout_order_received_url();
	                } else {
	                	$redirect = add_query_arg( 'key', $pedido->order_key, add_query_arg( 'order', $pedido->id, get_permalink( get_option( 'woocommerce_thanks_page_id' ) ) ) );
	                }

			    	wp_redirect( $redirect );
			    	
					header("HTTP/1.0 200 OK");
				}
			}
		}
		
		
		
		/**
		 * HTML form to simple integration
		 *
		 * @see https://www.moip.com.br/AdmCheckout.do?method=manual
		 * @param Object $order
		 */
		public function form_html( $order ) {

			// isn't Brazil?
			if ( !$this->is_valid_currency() ) {
				return;
			}

			$order->billing_phone = str_replace(array('(', '-', ' ', ')'), '', $order->billing_phone);
			$ddd = substr($order->billing_phone, 0, 2 );
			$telefone = substr($order->billing_phone, 2 );

			/**
			 * URL fix
			 * no support to query string
			 * site.com/order-received/?PagamentoDigitalretorno=ID
			 */
			//$urlReturn = get_permalink(get_option('woocommerce_thanks_page_id'))  .'?MoIPretorno='. $order->id;

			
			// no support to query string
			if ( method_exists( $order, 'get_checkout_order_received_url' ) ) {
            	$url_retorno = $order->get_checkout_order_received_url();
            } else {
            	$url_retorno = add_query_arg( 'key', $order->order_key, add_query_arg( 'order', $order->id, get_permalink( get_option( 'woocommerce_thanks_page_id' ) ) ) );
            }
			

			$item_loop = 1;

			$html  = '
				<!-- Store -->
				<input name="id_carteira" type="hidden" value="'. $this->account .'" />
				<input name="id_transacao" type="hidden" value="'. $order->id .'" />
				<input name="descricao" type="hidden" value="Pagamento do pedido Nº '. $order->id .'" />
				<input name="url_retorno" type="hidden" value="'. $urlReturn .'" />

				<!-- Products -->
				<input name="nome" type="hidden" value="Pagamento do pedido '. $order->id .'" />
				<input name="valor" type="hidden" value="'. str_replace( '.', '', $order->order_total ) .'" />

				<!-- Customer -->
				<input name="pagador_nome" type="hidden" value="'. $order->billing_first_name ." ". $order->billing_last_name .'" />
				<input name="pagador_email" type="hidden" value="'. $order->billing_email .'" />
				<input name="pagador_pais" type="hidden" value="Brasil" />
				';

			return $html;
		}
		
		
		
		


		/**
		 * Add an error message
		 */
		public function add_error( $message ) {
			if ( function_exists( 'wc_add_notice' ) ) {
				wc_add_notice( $message, 'error' );
			} else {
				global $woocommerce;

				$woocommerce->add_error( $message );
			}
		}
	
		
		

	}

	load_plugin_textdomain('WC_MoIP', false, basename( dirname( __FILE__ ) ) . '/languages' );

}

function add_moip_gateway( $methods ) {
	$methods[] = 'WC_Gateway_Moip'; return $methods;
}
